@@ -15,22 +15,20 @@
 
 #include "support/test_config.h"
 
-#include _PSTL_TEST_HEADER(execution)
-
-#include <oneapi/dpl/pstl/onedpl_config.h>
-
-#include "support/utils.h"
+#include _PSTL_TEST_HEADER(version)
 
 #if !__has_include(<execution>)
-static_assert(_PSTL_VERSION == 11000, "");
-static_assert(_PSTL_VERSION_MAJOR == 11, "");
-static_assert(_PSTL_VERSION_MINOR == 00, "");
-static_assert(_PSTL_VERSION_PATCH == 0, "");
+static_assert(_PSTL_VERSION == 14000);
+static_assert(_PSTL_VERSION_MAJOR == 14);
+static_assert(_PSTL_VERSION_MINOR == 00);
+static_assert(_PSTL_VERSION_PATCH == 0);
 #endif
 
-static_assert(ONEDPL_VERSION_MAJOR == 2022, "");
-static_assert(ONEDPL_VERSION_MINOR == 0, "");
-static_assert(ONEDPL_VERSION_PATCH == 0, "");
+static_assert(ONEDPL_VERSION_MAJOR == 2022);
+static_assert(ONEDPL_VERSION_MINOR == 7);
+static_assert(ONEDPL_VERSION_PATCH == 0);
+
+#include "support/utils.h"
 
 int main() {
 
