@@ -84,7 +84,7 @@ struct test_replace
     }
 
     template <typename T, typename Iterator1>
-    typename ::std::enable_if<::std::is_same<T, copy_int>::value, bool>::type_t
+    ::std::enable_if_t<::std::is_same_v<T, copy_int>, bool>
     check(Iterator1 b, Iterator1 e)
     {
         return ::std::all_of(b, e, [](const copy_int& elem) { return elem.copied_times == 0; });
@@ -122,16 +122,7 @@ test(Pred pred)
     Sequence<T2> expected(max_len);
     Sequence<T2> actual(max_len);
 
-    Sequence<T2> data(max_len, [&value](::std::size_t i) {
-        if (i % 3 == 2)
-        {
-            return T1(i);
-        }
-        else
-        {
-            return value;
-        }
-    });
+    Sequence<T2> data(max_len, [=](::std::size_t i) { return i % 3 == 2 ? T1(i) : value; });
 
     for (::std::size_t len = 0; len < max_len; len = len <= 16 ? len + 1 : ::std::size_t(3.1415 * len))
     {
