@@ -174,6 +174,18 @@ Equal(std::int32_t x, std::int32_t y)
     return x == y;
 }
 
+template <typename T, typename Compare>
+bool check_by_predicate(T t1, T t2, Compare c)
+{
+    return !c(t2, t1);
+}
+
+template <typename T>
+bool check_by_predicate(T t1, T t2)
+{
+    return !(t2 < t1);
+}
+
 template <typename InputIterator, typename OutputIterator1, typename OutputIterator2, typename Size>
 void
 copy_data(InputIterator first, OutputIterator1 expected_first, OutputIterator1 expected_last, OutputIterator2 tmp_first,
@@ -193,14 +205,21 @@ sort_data(Params&& ...params)
         ::std::sort(::std::forward<Params>(params)...);
 }
 
-template <typename OutputIterator1, typename OutputIterator2, typename Size>
+template <typename OutputIterator1, typename OutputIterator2, typename Size, typename... Compare>
 void
-check_results(OutputIterator1 expected_first, OutputIterator2 tmp_first, Size n, const char* error_msg)
+check_results(OutputIterator1 expected_first, OutputIterator2 tmp_first, Size n, const char* error_msg, Compare... compare)
 {
+    auto pred = *tmp_first;
     for (size_t i = 0; i < n; ++i, ++expected_first, ++tmp_first)
     {
         // Check that expected[i] is equal to tmp[i]
         EXPECT_TRUE(Equal(*expected_first, *tmp_first), error_msg);
+        // Compare with the previous element using the predicate
+        if (i > 1 && i < n-1) // first and last were not sorted
+        {
+            EXPECT_TRUE(check_by_predicate(pred, *tmp_first, compare...), error_msg);
+        }
+        pred = *tmp_first;
     }
 }
 
@@ -235,7 +254,7 @@ test_usm(Policy&& exec, OutputIterator tmp_first, OutputIterator tmp_last, Outpu
     // check result
     dt_helper.retrieve_data(_it_from);
 
-    check_results(expected_first, tmp_first, n, "wrong result from sort without predicate #2");
+    check_results(expected_first, tmp_first, n, "wrong result from sort without predicate #2", compare...);
 
     const std::int32_t count1 = KeyCount;
     EXPECT_EQ(count0, count1, "key cleanup error");
@@ -243,9 +262,16 @@ test_usm(Policy&& exec, OutputIterator tmp_first, OutputIterator tmp_last, Outpu
 #endif // _PSTL_SYCL_TEST_USM
 #endif // TEST_DPCPP_BACKEND_PRESENT
 
+// Additional check for std::execution::par_unseq is required because standard execution policy is
+// not a host execution policy in terms of oneDPL and the eligible overload of run_test would not be found
+// while testing PSTL offload
 template <typename Policy, typename InputIterator, typename OutputIterator, typename OutputIterator2, typename Size,
           typename... Compare>
-oneapi::dpl::__internal::__enable_if_host_execution_policy<Policy, void>
+std::enable_if_t<oneapi::dpl::__internal::__is_host_execution_policy<std::decay_t<Policy>>::value
+#if __SYCL_PSTL_OFFLOAD__
+                 || std::is_same_v<std::decay_t<Policy>, std::execution::parallel_unsequenced_policy>
+#endif
+                 >
 run_test(Policy&& exec, OutputIterator tmp_first, OutputIterator tmp_last, OutputIterator2 expected_first,
          OutputIterator2 expected_last, InputIterator first, InputIterator /*last*/, Size n, Compare ...compare)
 {
@@ -257,7 +283,7 @@ run_test(Policy&& exec, OutputIterator tmp_first, OutputIterator tmp_last, Outpu
     const std::int32_t count0 = KeyCount;
     sort_data(::std::forward<Policy>(exec), tmp_first + 1, tmp_last - 1, compare...);
 
-    check_results(expected_first, tmp_first, n, "wrong result from sort without predicate #1");
+    check_results(expected_first, tmp_first, n, "wrong result from sort without predicate #1", compare...);
 
     const std::int32_t count1 = KeyCount;
     EXPECT_EQ(count0, count1, "key cleanup error");
@@ -267,7 +293,7 @@ run_test(Policy&& exec, OutputIterator tmp_first, OutputIterator tmp_last, Outpu
 #if _PSTL_SYCL_TEST_USM
 template <typename Policy, typename InputIterator, typename OutputIterator, typename OutputIterator2, typename Size,
           typename... Compare>
-oneapi::dpl::__internal::__enable_if_hetero_execution_policy<Policy, void>
+oneapi::dpl::__internal::__enable_if_hetero_execution_policy<Policy>
 run_test(Policy&& exec, OutputIterator tmp_first, OutputIterator tmp_last, OutputIterator2 expected_first,
             OutputIterator2 expected_last, InputIterator first, InputIterator last, Size n, Compare ...compare)
 {
@@ -285,10 +311,10 @@ template <typename T>
 struct test_sort_op
 {
     template <typename Policy, typename InputIterator, typename OutputIterator, typename OutputIterator2, typename Size,
-              typename ...Compare>
-    typename ::std::enable_if<TestUtils::is_base_of_iterator_category<::std::random_access_iterator_tag, InputIterator>::value
-                          && (TestUtils::can_use_default_less_operator<T>::value || sizeof...(Compare) > 0),
-                              void>::type
+              typename... Compare>
+    ::std::enable_if_t<
+        TestUtils::is_base_of_iterator_category_v<::std::random_access_iterator_tag, InputIterator> &&
+            (TestUtils::can_use_default_less_operator_v<T> || sizeof...(Compare) > 0)>
     operator()(Policy&& exec, OutputIterator tmp_first, OutputIterator tmp_last, OutputIterator2 expected_first,
                OutputIterator2 expected_last, InputIterator first, InputIterator last, Size n, Compare ...compare)
     {
@@ -297,10 +323,10 @@ struct test_sort_op
     }
 
     template <typename Policy, typename InputIterator, typename OutputIterator, typename OutputIterator2, typename Size,
-              typename ...Compare>
-    typename ::std::enable_if<!TestUtils::is_base_of_iterator_category<::std::random_access_iterator_tag, InputIterator>::value
-                          || !(TestUtils::can_use_default_less_operator<T>::value || sizeof...(Compare) > 0),
-                              void>::type
+              typename... Compare>
+    ::std::enable_if_t<
+        !TestUtils::is_base_of_iterator_category_v<::std::random_access_iterator_tag, InputIterator> ||
+            !(TestUtils::can_use_default_less_operator_v<T> || sizeof...(Compare) > 0)>
     operator()(Policy&& /* exec */, OutputIterator /* tmp_first */, OutputIterator /* tmp_last */,
                OutputIterator2 /* expected_first */, OutputIterator2 /* expected_last */, InputIterator /* first */,
                InputIterator /* last */, Size /* n */, Compare .../*compare*/)
@@ -320,21 +346,21 @@ test_default_name_gen(Convert convert, size_t n)
     TestUtils::Sequence<T> in(n + 2, [=](size_t k) { return convert(k, rand() % (2 * n + 1)); });
     TestUtils::Sequence<T> expected(in);
     TestUtils::Sequence<T> tmp(in);
-    auto my_policy = oneapi::dpl::execution::make_device_policy(TestUtils::get_test_queue());
-    
+    auto my_policy = TestUtils::make_device_policy(TestUtils::get_test_queue());
+
     TestUtils::iterator_invoker<::std::random_access_iterator_tag, /*IsReverse*/ ::std::false_type>()(
                 my_policy, test_sort_op<T>(), tmp.begin(), tmp.end(), expected.begin(), expected.end(), in.begin(), in.end(),
                     in.size(), ::std::greater<void>());
     TestUtils::iterator_invoker<::std::random_access_iterator_tag, /*IsReverse*/ ::std::false_type>()(
                 my_policy, test_sort_op<T>(), tmp.begin(), tmp.end(), expected.begin(), expected.end(), in.begin(), in.end(),
                     in.size(), ::std::less<void>());
-                    
+
 }
 #    endif //__SYCL_UNNAMED_LAMBDA__
 #endif //TEST_DPCPP_BACKEND_PRESENT
 
 
-template <typename T, typename Compare, typename Convert>
+template <::std::size_t CallNumber, typename T, typename Compare, typename Convert>
 void
 test_sort(Compare compare, Convert convert)
 {
@@ -347,12 +373,12 @@ test_sort(Compare compare, Convert convert)
         TestUtils::Sequence<T> expected(in);
         TestUtils::Sequence<T> tmp(in);
 #ifdef _PSTL_TEST_WITHOUT_PREDICATE
-        TestUtils::invoke_on_all_policies<0>()(test_sort_op<T>(), tmp.begin(), tmp.end(), expected.begin(),
-                                               expected.end(), in.begin(), in.end(), in.size());
+        TestUtils::invoke_on_all_policies<CallNumber>()(test_sort_op<T>(), tmp.begin(), tmp.end(), expected.begin(),
+                                                        expected.end(), in.begin(), in.end(), in.size());
 #endif // _PSTL_TEST_WITHOUT_PREDICATE
 #ifdef _PSTL_TEST_WITH_PREDICATE
-        TestUtils::invoke_on_all_policies<1>()(test_sort_op<T>(), tmp.begin(), tmp.end(), expected.begin(),
-                                               expected.end(), in.begin(), in.end(), in.size(), compare);
+        TestUtils::invoke_on_all_policies<CallNumber + 1>()(test_sort_op<T>(), tmp.begin(), tmp.end(), expected.begin(),
+                                                            expected.end(), in.begin(), in.end(), in.size(), compare);
 #endif // _PSTL_TEST_WITH_PREDICATE
     }
 }
@@ -373,6 +399,15 @@ struct test_non_const
     }
 };
 
+struct NonConstCmp
+{
+    template<typename T, typename U>
+    bool operator()(T x, U y)
+    {
+        return x < y;
+    }
+};
+
 int
 main()
 {
@@ -391,17 +426,48 @@ main()
 
 #if !TEST_DPCPP_BACKEND_PRESENT
         // ParanoidKey has atomic increment in ctors. It's not allowed in kernel
-        test_sort<ParanoidKey>(KeyCompare(TestUtils::OddTag()),
-                               [](size_t k, size_t val) { return ParanoidKey(k, val, TestUtils::OddTag()); });
+        test_sort<0, ParanoidKey>(KeyCompare(TestUtils::OddTag()),
+                                  [](size_t k, size_t val) { return ParanoidKey(k, val, TestUtils::OddTag()); });
 #endif // !TEST_DPCPP_BACKEND_PRESENT
 
 #if !ONEDPL_FPGA_DEVICE
-        test_sort<TestUtils::float32_t>([](TestUtils::float32_t x, TestUtils::float32_t y) { return x < y; },
-                                        [](size_t, size_t val) { return TestUtils::float32_t(val); });
+        test_sort<10, TestUtils::float32_t>([](TestUtils::float32_t x, TestUtils::float32_t y) { return x < y; },
+                                            [](size_t k, size_t val)
+                                            { return TestUtils::float32_t(val) * (k % 2 ? 1 : -1); });
+
+        test_sort<20, unsigned char>([](unsigned char x, unsigned char y)
+                                     { return x > y; }, // Reversed so accidental use of < will be detected.
+                                     [](size_t k, size_t val) { return (unsigned char)val; });
+
+        test_sort<30, unsigned char>(NonConstCmp{}, [](size_t k, size_t val) { return (unsigned char)val; });
+
 #endif // !ONEDPL_FPGA_DEVICE
-        test_sort<std::int32_t>(
-            [](std::int32_t x, std::int32_t y) { return x > y; }, // Reversed so accidental use of < will be detected.
-            [](size_t, size_t val) { return std::int32_t(val); });
+        test_sort<40, std::int32_t>([](std::int32_t x, std::int32_t y)
+                                    { return x > y; }, // Reversed so accidental use of < will be detected.
+                                    [](size_t k, size_t val) { return std::int32_t(val) * (k % 2 ? 1 : -1); });
+
+        test_sort<50, std::int16_t>(
+            std::greater<std::int16_t>(),
+            [](size_t k, size_t val) {
+            return std::int16_t(val) * (k % 2 ? 1 : -1); });
+
+#if TEST_DPCPP_BACKEND_PRESENT
+        auto convert = [](size_t k, size_t val) {
+            constexpr std::uint16_t mask = 0xFFFFu;
+            std::uint16_t raw = std::uint16_t(val & mask);
+            // Avoid NaN values, because they need a custom comparator due to: (x < NaN = false) and (NaN < x = false).
+            constexpr std::uint16_t exp_mask = 0x7C00u;
+            constexpr std::uint16_t frac_mask = 0x03FFu;
+            bool is_nan = ((raw & exp_mask) == exp_mask) && ((raw & frac_mask) > 0);
+            if (is_nan)
+            {
+                constexpr std::uint16_t smallest_exp_bit = 0x0400u;
+                raw = raw & (~smallest_exp_bit); // flip the smallest exponent bit
+            }
+            return sycl::bit_cast<sycl::half>(raw);
+        };
+        test_sort<60, sycl::half>(std::greater<sycl::half>(), convert);
+#endif
     }
 
 #if TEST_DPCPP_BACKEND_PRESENT
