@@ -13,8 +13,8 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef __UTILS_H
-#define __UTILS_H
+#ifndef _UTILS_H
+#define _UTILS_H
 
 // File contains common utilities that tests rely on
 
@@ -29,9 +29,16 @@
 #include <cstring>
 #include <iostream>
 #include <iterator>
+#include <cmath>
+#include <complex>
+#include <type_traits>
 #include <memory>
 #include <sstream>
 #include <vector>
+#include <tuple>
+#include <random>
+#include <limits>
+#include <cassert>
 
 #include "utils_const.h"
 #include "iterator_utils.h"
@@ -40,6 +47,7 @@
 
 #if TEST_DPCPP_BACKEND_PRESENT
 #    include "utils_sycl.h"
+#    include "oneapi/dpl/experimental/kt/kernel_param.h"
 #endif
 
 namespace TestUtils
@@ -94,11 +102,22 @@ expect(bool expected, bool condition, const char* file, std::int32_t line, const
 
 // Do not change signature to const T&.
 // Function must be able to detect const differences between expected and actual.
-template <typename T>
+template <typename T1, typename T2>
 void
-expect_equal_val(T& expected, T& actual, const char* file, std::int32_t line, const char* message)
+expect_equal_val(const T1& expected, const T2& actual, const char* file, std::int32_t line, const char* message)
 {
-    if (!(expected == actual))
+    using T = std::common_type_t<T1, T2>;
+
+    bool __is_error = false;
+    if constexpr (std::is_floating_point_v<T>)
+    {
+        const auto eps = std::numeric_limits<T>::epsilon();
+        __is_error = std::fabs(T(expected) - T(actual)) >= eps;
+    }
+    else
+        __is_error = !(T(expected) == T(actual));
+
+    if (__is_error)
     {
         ::std::stringstream outstr;
         outstr << "error at " << file << ":" << line << " - " << message << ", expected " << expected << " got "
@@ -160,6 +179,18 @@ expect_equal(Iterator1 expected_first, Iterator2 actual_first, Size n, const cha
     }
 }
 
+template <typename T1, typename T2>
+bool
+check_data(const T1* device_iter, const T2* host_iter, int N)
+{
+    for (int i = 0; i < N; ++i)
+    {
+        if (*(host_iter + i) != *(device_iter + i))
+            return false;
+    }
+    return true;
+}
+
 struct MemoryChecker
 {
     // static counters and state tags
@@ -269,19 +300,19 @@ struct MemoryChecker
     }
 };
 
-::std::atomic<::std::size_t> MemoryChecker::alive_object_counter{0};
+inline ::std::atomic<::std::size_t> MemoryChecker::alive_object_counter{0};
 
-::std::ostream&
+inline ::std::ostream&
 operator<<(::std::ostream& os, const MemoryChecker& val)
 {
     return (os << val.value());
 }
-bool
+inline bool
 operator==(const MemoryChecker& v1, const MemoryChecker& v2)
 {
     return v1.value() == v2.value();
 }
-bool
+inline bool
 operator<(const MemoryChecker& v1, const MemoryChecker& v2)
 {
     return v1.value() < v2.value();
@@ -313,12 +344,10 @@ HashBits(size_t i, size_t bits)
 
 // Stateful unary op
 template <typename T, typename U>
-class Complement
+struct Complement
 {
-    std::int32_t val;
+    std::int32_t val = 1;
 
-  public:
-    Complement(T v) : val(v) {}
     U
     operator()(const T& x) const
     {
@@ -326,6 +355,48 @@ class Complement
     }
 };
 
+struct ComplementZip
+{
+    std::int32_t val = 1;
+
+    template<typename T>
+    auto
+    operator()(const oneapi::dpl::__internal::tuple<T&>& t) const
+    {
+        return oneapi::dpl::__internal::tuple<T>(val - std::get<0>(t));
+    }
+};
+
+template <typename In1, typename In2, typename Out>
+class TheOperation
+{
+    Out val;
+
+  public:
+    TheOperation(Out v) : val(v) {}
+    Out
+    operator()(const In1& x, const In2& y) const
+    {
+        return Out(val + x - y);
+    }
+};
+
+template <typename Out>
+class TheOperationZip
+{
+    Out val;
+
+  public:
+    TheOperationZip(Out v) : val(v) {}
+
+    template <typename T1, typename T2>
+    auto
+    operator()(const oneapi::dpl::__internal::tuple<T1&>& t1, const oneapi::dpl::__internal::tuple<T2&>& t2) const
+    {
+        return oneapi::dpl::__internal::tuple<Out>(val + std::get<0>(t1) - std::get<0>(t2));
+    }
+};
+
 // Tag used to prevent accidental use of converting constructor, even if use is explicit.
 struct OddTag
 {
@@ -523,24 +594,23 @@ class Wrapper
 {
   public:
     Wrapper()
+        : my_field(::std::make_shared<T>())
     {
-        my_field = ::std::shared_ptr<T>(new T());
         ++my_count;
     }
     Wrapper(const T& input)
+        : my_field(::std::make_shared<T>(input))
     {
-        my_field = ::std::shared_ptr<T>(new T(input));
         ++my_count;
     }
     Wrapper(const Wrapper& input)
+        : my_field(input.my_field)
     {
-        my_field = input.my_field;
         ++my_count;
     }
     Wrapper(Wrapper&& input)
+        : my_field(::std::move(input.my_field))
     {
-        my_field = input.my_field;
-        input.my_field = nullptr;
         ++move_count;
     }
     Wrapper&
@@ -552,8 +622,7 @@ class Wrapper
     Wrapper&
     operator=(Wrapper&& input)
     {
-        my_field = input.my_field;
-        input.my_field = nullptr;
+        my_field = ::std::move(input.my_field);
         ++move_count;
         return *this;
     }
@@ -635,7 +704,7 @@ transform_reduce_serial(InputIterator first, InputIterator last, T init, BinaryO
     return init;
 }
 
-int
+inline int
 done(int is_done = 1)
 {
     if (is_done)
@@ -695,6 +764,238 @@ struct can_use_default_less_operator<T, decltype(::std::declval<T>() < ::std::de
 {
 };
 
+template <typename T>
+inline constexpr bool can_use_default_less_operator_v = can_use_default_less_operator<T>::value;
+
+// An arbitrary binary predicate to simulate a predicate the user providing
+// a custom predicate.
+template <typename _Tp>
+struct UserBinaryPredicate
+{
+    bool
+    operator()(const _Tp& __x, const _Tp& __y) const
+    {
+        using KeyT = ::std::decay_t<_Tp>;
+        return __y != KeyT(1);
+    }
+};
+
+template <typename T>
+struct MatrixPoint
+{
+    T m;
+    T n;
+    MatrixPoint() = default;
+    MatrixPoint(T m, T n = {}) : m(m), n(n) {}
+    bool
+    operator==(const MatrixPoint& other) const
+    {
+        return m == other.m && n == other.n;
+    }
+    bool
+    operator!=(const MatrixPoint& other) const
+    {
+        return !(*this == other);
+    }
+    MatrixPoint
+    operator+(const MatrixPoint& other) const
+    {
+        return MatrixPoint(m + other.m, n + other.n);
+    }
+};
+
+template <typename T>
+std::ostream&
+operator<<(std::ostream& os, MatrixPoint<T> matrix_point)
+{
+    return os << "(" << matrix_point.m << ", " << matrix_point.n << ")";
+}
+
+template <typename _Tp>
+struct MaxFunctor
+{
+    _Tp
+    operator()(const _Tp& __x, const _Tp& __y) const
+    {
+        return (__x < __y) ? __y : __x;
+    }
+};
+
+template <typename _Tp>
+struct MaxFunctor<MatrixPoint<_Tp>>
+{
+    auto
+    sum(const MatrixPoint<_Tp>& __x) const
+    {
+        return __x.m + __x.n;
+    }
+    MatrixPoint<_Tp>
+    operator()(const MatrixPoint<_Tp>& __x, const MatrixPoint<_Tp>& __y) const
+    {
+        return (sum(__x) < sum(__y)) ? __y : __x;
+    }
+};
+
+template <typename _Tp>
+struct MaxAbsFunctor;
+
+// A modification of the functor we use in reduce_by_segment
+template <typename _Tp>
+struct MaxAbsFunctor<MatrixPoint<_Tp>>
+{
+    auto
+    abs_sum(const MatrixPoint<_Tp>& __x) const
+    {
+        return std::sqrt(__x.m * __x.m + __x.n * __x.n);
+    }
+    MatrixPoint<_Tp>
+    operator()(const MatrixPoint<_Tp>& __x, const MatrixPoint<_Tp>& __y) const
+    {
+        return (abs_sum(__x) < abs_sum(__y)) ? abs_sum(__y) : abs_sum(__x);
+    }
+};
+
+struct TupleAddFunctor
+{
+    template <typename Tup1, typename Tup2>
+    auto
+    operator()(const Tup1& lhs, const Tup2& rhs) const
+    {
+        using ::std::get;
+        Tup1 tup_sum = ::std::make_tuple(get<0>(lhs) + get<0>(rhs), get<1>(lhs) + get<1>(rhs));
+        return tup_sum;
+    }
+};
+
+struct _Identity
+{
+    template< class T >
+    constexpr T&& operator()(T&& t) const noexcept
+    {
+        return std::forward<T>(t);
+    }
+};
+
+struct _ZipIteratorAdapter
+{
+    template< class T >
+    constexpr auto operator()(T&& t) const noexcept
+    {
+        return dpl::make_zip_iterator(std::forward<T>(t));
+    }
+};
+
+#if TEST_DPCPP_BACKEND_PRESENT
+template <typename Iter, typename ValueType = std::decay_t<typename std::iterator_traits<Iter>::value_type>>
+using __default_alloc_vec_iter = typename std::vector<ValueType>::iterator;
+
+template <typename Iter, typename ValueType = std::decay_t<typename std::iterator_traits<Iter>::value_type>>
+using __usm_shared_alloc_vec_iter =
+    typename std::vector<ValueType, typename sycl::usm_allocator<ValueType, sycl::usm::alloc::shared>>::iterator;
+
+template <typename Iter, typename ValueType = std::decay_t<typename std::iterator_traits<Iter>::value_type>>
+using __usm_host_alloc_vec_iter =
+    typename std::vector<ValueType, typename sycl::usm_allocator<ValueType, sycl::usm::alloc::host>>::iterator;
+
+// Evaluates to true if the provided type is an iterator with a value_type and if the implementation of a
+// std::vector<value_type, Alloc>::iterator can be distinguished between three different allocators, the
+// default, usm_shared, and usm_host. If all are distinct, it is very unlikely any non-usm based allocator
+// could be confused with a usm allocator.
+template <typename Iter>
+constexpr bool __vector_impl_distinguishes_usm_allocator_from_default_v =
+    !std::is_same_v<__default_alloc_vec_iter<Iter>, __usm_shared_alloc_vec_iter<Iter>> &&
+    !std::is_same_v<__default_alloc_vec_iter<Iter>, __usm_host_alloc_vec_iter<Iter>> &&
+    !std::is_same_v<__usm_host_alloc_vec_iter<Iter>, __usm_shared_alloc_vec_iter<Iter>>;
+
+#endif //TEST_DPCPP_BACKEND_PRESENT
+
+////////////////////////////////////////////////////////////////////////////////
+// Implementation of create_new_policy for all policies (host + hetero)
+template <typename Policy>
+using __is_able_to_create_new_policy =
+#if TEST_DPCPP_BACKEND_PRESENT
+    oneapi::dpl::__internal::__is_hetero_execution_policy<::std::decay_t<Policy>>;
+#else
+    ::std::false_type;
+#endif // TEST_DPCPP_BACKEND_PRESENT
+
+#if TEST_DPCPP_BACKEND_PRESENT
+template <typename _NewKernelName, typename Policy, ::std::enable_if_t<__is_able_to_create_new_policy<Policy>::value, int> = 0>
+auto
+create_new_policy(Policy&& policy)
+{
+    return TestUtils::make_new_policy<_NewKernelName>(::std::forward<Policy>(policy));
+}
+#endif // TEST_DPCPP_BACKEND_PRESENT
+
+template <typename _NewKernelName, typename Policy, ::std::enable_if_t<!__is_able_to_create_new_policy<Policy>::value, int> = 0>
+auto
+create_new_policy(Policy&& policy)
+{
+    return ::std::forward<Policy>(policy);
+}
+
+template <int idx, typename Policy>
+auto
+create_new_policy_idx(Policy&& policy)
+{
+#if TEST_DPCPP_BACKEND_PRESENT
+    return create_new_policy<TestUtils::new_kernel_name<Policy, idx>>(::std::forward<Policy>(policy));
+#else
+    return ::std::forward<Policy>(policy);
+#endif
+}
+
+#if TEST_DPCPP_BACKEND_PRESENT
+template <typename KernelName, int idx>
+struct __kernel_name_with_idx
+{
+};
+
+template <int idx, typename KernelParams>
+constexpr auto
+get_new_kernel_params(KernelParams)
+{
+    return oneapi::dpl::experimental::kt::kernel_param<
+        KernelParams::data_per_workitem, KernelParams::workgroup_size,
+        __kernel_name_with_idx<typename KernelParams::kernel_name, idx>>{};
+}
+#endif //TEST_DPCPP_BACKEND_PRESENT
+
+template <typename T>
+typename std::enable_if_t<std::is_arithmetic_v<T>>
+generate_arithmetic_data(T* input, std::size_t size, std::uint32_t seed)
+{
+    std::default_random_engine gen{seed};
+    // The values beyond the threshold (75%) are duplicates of the values within the threshold
+    std::size_t unique_threshold = 75 * size / 100;
+    if constexpr (std::is_integral_v<T>)
+    {
+        // no uniform_int_distribution for chars
+        using GenT = std::conditional_t<sizeof(T) < sizeof(short), int, T>;
+        std::uniform_int_distribution<GenT> dist(std::numeric_limits<T>::lowest(), std::numeric_limits<T>::max());
+        std::generate(input, input + unique_threshold, [&] { return T(dist(gen)); });
+    }
+    else
+    {
+        // log2 - exp2 transformation allows generating floating point values,
+        // which distribution resembles uniform distribution of their bit representation
+        // This is useful for checking different cases of radix sort
+        std::uniform_real_distribution<T> dist_real(std::numeric_limits<T>::min(), log2(std::numeric_limits<T>::max()));
+        std::uniform_int_distribution<int> dist_binary(0, 1);
+        auto randomly_signed_real = [&dist_real, &dist_binary, &gen]()
+        {
+            auto v = exp2(dist_real(gen));
+            return dist_binary(gen) == 0 ? v : -v;
+        };
+        std::generate(input, input + unique_threshold, [&] { return randomly_signed_real(); });
+    }
+    assert(unique_threshold >= size/2 && unique_threshold < size);
+    for (uint32_t i = 0, j = unique_threshold; j < size; ++i, ++j)
+    {
+        input[j] = input[i];
+    }
+}
 } /* namespace TestUtils */
 
-#endif // __UTILS_H
+#endif // _UTILS_H
