@@ -13,21 +13,16 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "support/test_config.h"
+
 #include <oneapi/dpl/cmath>
 
-#include "support/test_config.h"
 #include "support/utils.h"
 #include "support/utils_invoke.h"
 
 #include <iostream>
 #include <vector>
 
-#if TEST_DPCPP_BACKEND_PRESENT
-#include <CL/sycl.hpp>
-
-constexpr sycl::access::mode sycl_read = sycl::access::mode::read;
-constexpr sycl::access::mode sycl_write = sycl::access::mode::write;
-
 template <typename KernelClass, typename Function, typename ValueType>
 void
 test(Function fnc, const std::vector<ValueType>& args, const char* message)
@@ -41,12 +36,12 @@ test(Function fnc, const std::vector<ValueType>& args, const char* message)
 
     // Evaluate results in Kernel
     {
-        sycl::queue deviceQueue(TestUtils::async_handler);
+        sycl::queue deviceQueue = TestUtils::get_test_queue();
 
         if (!TestUtils::has_type_support<ValueType>(deviceQueue.get_device()))
         {
-            std::cout << deviceQueue.get_device().template get_info<sycl::info::device::name>()
-                      << " does not support " << typeid(ValueType).name() << " type,"
+            std::cout << deviceQueue.get_device().template get_info<sycl::info::device::name>() << " does not support "
+                      << typeid(ValueType).name() << " type,"
                       << " affected test case have been skipped" << std::endl;
             return;
         }
@@ -54,18 +49,14 @@ test(Function fnc, const std::vector<ValueType>& args, const char* message)
         sycl::buffer<ValueType> buffer1(args.data(), args_count);
         sycl::buffer<ValueType> buffer2(output.data(), args_count);
 
-        deviceQueue.submit(
-            [&](sycl::handler& cgh)
-            {
-                auto in = buffer1.template get_access<sycl_read>(cgh);
-                auto out = buffer2.template get_access<sycl_write>(cgh);
-                cgh.single_task<KernelClass>(
-                    [=]()
-                    {
-                        for (size_t i = 0; i < args_count; ++i)
-                            out[i] = fnc(in[i]);
-                    });
+        deviceQueue.submit([&](sycl::handler& cgh) {
+            auto in = buffer1.template get_access<sycl::access::mode::read>(cgh);
+            auto out = buffer2.template get_access<sycl::access::mode::write>(cgh);
+            cgh.single_task<KernelClass>([=]() {
+                for (size_t i = 0; i < args_count; ++i)
+                    out[i] = fnc(in[i]);
             });
+        });
     }
 
     // Check results: compare resuls evaluated in Kernel and on host
@@ -78,13 +69,9 @@ test(Function fnc, const std::vector<ValueType>& args, const char* message)
 
 class Test;
 
-#endif // TEST_DPCPP_BACKEND_PRESENT
-
 int
 main()
 {
-#if TEST_DPCPP_BACKEND_PRESENT
-
     // functions from https://en.cppreference.com/w/cpp/numeric/math/nearbyint :
     //     float  nearbyint (float arg);
     //     float  nearbyintf(float arg);
@@ -107,7 +94,5 @@ main()
     const std::vector<double> f_args_double = {+2.3, +2.5, +3.5, -2.3, -2.5, -3.5};
     test<TestUtils::unique_kernel_name<Test, 2>>(f_nearbyint_double, f_args_double, "double nearbyint(double)");
 
-#endif // TEST_DPCPP_BACKEND_PRESENT
-
     return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT);
 }
