@@ -17,13 +17,14 @@
 //
 // Common functionality for conformance tests
 
-#ifndef DPSTD_RANDOM_CONFORMANCE_TESTS_COMMON
-#define DPSTD_RANDOM_CONFORMANCE_TESTS_COMMON
+#ifndef _DPSTD_RANDOM_CONFORMANCE_TESTS_COMMON_HPP
+#define _DPSTD_RANDOM_CONFORMANCE_TESTS_COMMON_HPP
 
 #include <vector>
-#include <CL/sycl.hpp>
 #include <random>
 
+#include "support/utils.h"
+
 constexpr auto REF_SAMPLE_ID = 9999;
 
 template<class Engine, int NGenSamples, int NElemsInResultType>
@@ -65,4 +66,4 @@ typename Engine::scalar_type test(sycl::queue& queue) {
     return dpstd_samples[REF_SAMPLE_ID];
 }
 
-#endif // ifndef DPSTD_RANDOM_CONFORMANCE_TESTS_COMMON
\ No newline at end of file
+#endif // _DPSTD_RANDOM_CONFORMANCE_TESTS_COMMON_HPP
