@@ -19,12 +19,13 @@
 
 #include "support/utils.h"
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
+#include <oneapi/dpl/random>
+static_assert(ONEDPL_HAS_RANDOM_NUMBERS >= 202409L);
+
 #include <iostream>
 #include <cmath>
 #include <vector>
-#include <CL/sycl.hpp>
-#include <oneapi/dpl/random>
 
 constexpr auto SEED = 777;
 constexpr auto N_GEN = 960;
@@ -140,8 +141,7 @@ check_params(oneapi::dpl::extreme_value_distribution<T>& distr)
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr,
-        oneapi::dpl::uniform_int_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::uniform_int_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{0, 10};
@@ -149,8 +149,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::uniform_real_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::uniform_real_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{1.5, 3.0};
@@ -158,8 +157,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::exponential_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::exponential_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{1.5};
@@ -167,8 +165,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::bernoulli_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::bernoulli_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{0.5};
@@ -176,8 +173,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::geometric_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::geometric_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{0.5};
@@ -185,8 +181,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::weibull_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::weibull_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{1.5, 3.0};
@@ -194,8 +189,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::lognormal_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::lognormal_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{1.5, 3.5};
@@ -203,8 +197,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::normal_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::normal_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{1.5, 3.5};
@@ -212,8 +205,7 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::cauchy_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::cauchy_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{1.5, 3.5};
@@ -221,14 +213,146 @@ make_param(typename Distr::param_type& params1, typename Distr::param_type& para
 }
 
 template <typename Distr>
-typename ::std::enable_if<::std::is_same<Distr, 
-        oneapi::dpl::extreme_value_distribution<typename Distr::result_type>>::value, void>::type
+::std::enable_if_t<::std::is_same_v<Distr, oneapi::dpl::extreme_value_distribution<typename Distr::result_type>>>
 make_param(typename Distr::param_type& params1, typename Distr::param_type& params2)
 {
     params1 = typename Distr::param_type{1.5, 3.5};
     params2 = typename Distr::param_type{-2, 10};
 }
 
+template <class Distr>
+std::int32_t
+check_input_output(Distr& distr)
+{
+    using params_type = typename Distr::scalar_type;
+    using result_type = typename Distr::result_type;
+
+    std::int32_t status = 0;
+
+    if constexpr (std::is_same_v<Distr, oneapi::dpl::bernoulli_distribution<result_type>>
+            || std::is_same_v<Distr, oneapi::dpl::geometric_distribution<result_type>>) {
+        double p = 0.5;
+
+        std::stringstream s;
+        s << p;
+
+        std::ostringstream out;
+        std::istringstream in(s.str());
+
+        in >> distr;
+        status += check_params(distr);
+
+        out << distr;
+        if (!(out.str() == s.str()))
+        {
+            status += 1;
+        }
+    }
+    else if constexpr (std::is_same_v<Distr, oneapi::dpl::exponential_distribution<result_type>>) {
+        params_type lambda{1.0};
+
+        std::stringstream s;
+        s << lambda;
+
+        std::ostringstream out;
+        std::istringstream in(s.str());
+
+        in >> distr;
+        status += check_params(distr);
+
+        out << distr;
+        if (!(out.str() == s.str()))
+        {
+            status += 1;
+        }
+    }
+    else if constexpr (std::is_same_v<Distr, oneapi::dpl::normal_distribution<result_type>> 
+                    || std::is_same_v<Distr, oneapi::dpl::lognormal_distribution<result_type>>) {
+        params_type mean{0.0};
+        params_type stddev{1.0};
+
+        std::stringstream s;
+        s << mean << ' ' << stddev;
+
+        bool flag_ = true;
+        params_type saved_ln_{1.2};
+        params_type saved_u2_{1.3};
+
+        s << ' ' << flag_ << ' ' << saved_ln_ << ' ' << saved_u2_;
+
+        std::ostringstream out;
+        std::istringstream in(s.str());
+
+        in >> distr;
+        status += check_params(distr);
+
+        out << distr;
+        if (!(out.str() == s.str()))
+        {
+            status += 1;
+        }
+    }
+    else if constexpr (std::is_same_v<Distr, oneapi::dpl::weibull_distribution<result_type>>) {
+        params_type a{1.0};
+        params_type b{1.0};
+
+        std::stringstream s;
+        s << a << ' ' << b;
+        
+        std::ostringstream out;
+        std::istringstream in(s.str());
+
+        in >> distr;
+        status += check_params(distr);
+
+        out << distr;
+        if (!(out.str() == s.str()))
+        {
+            status += 1;
+        }
+    }
+    else if constexpr (std::is_same_v<Distr, oneapi::dpl::uniform_int_distribution<result_type>>) {
+        params_type a{0};
+        params_type b = std::numeric_limits<params_type>::max();
+
+        std::stringstream s;
+        s << a << ' ' << b;
+
+        std::ostringstream out;
+        std::istringstream in(s.str());
+
+        in >> distr;
+        status += check_params(distr);
+
+        out << distr;
+        if (!(out.str() == s.str()))
+        {
+            status += 1;
+        }
+    }
+    else { // uniform_real_distribution, cauchy_distribution, extreme_value_distribution
+        params_type a{0.0};
+        params_type b{1.0};
+
+        std::stringstream s;
+        s << a << ' ' << b;
+
+        std::ostringstream out;
+        std::istringstream in(s.str());
+
+        in >> distr;
+        status += check_params(distr);
+
+        out << distr;
+        if (!(out.str() == s.str()))
+        {
+            status += 1;
+        }
+    }
+
+    return status;
+}
+
 template <class Distr>
 bool
 test_vec(sycl::queue& queue)
@@ -300,17 +424,44 @@ test(sycl::queue& queue)
 
     make_param<Distr>(params1, params2);
 
-    int sum = 0;
+    int status = 0;
 
     // Memory allocation
     typename Distr::scalar_type res[N_GEN];
 
     // Random number generation
     {
+        {
+            Distr _d1(2);
+            Distr _d2(2);
+            if (_d1 != _d2)
+            {
+                status += 1;
+                std::cout << "Error: d1 != d2" << std::endl;
+            }
+
+            status += check_input_output(_d1);
+
+            if (_d1 == _d2)
+            {
+                status += 1;
+                std::cout << "Error: d1 == d2" << std::endl;
+            }
+        }
+
         sycl::buffer<typename Distr::scalar_type> buffer(res, N_GEN);
 
         try
         {
+            queue.submit([&](sycl::handler& cgh) {
+                sycl::stream out(1024, 256, cgh);
+                cgh.single_task<>([=]() {
+                    Distr distr;
+                    out << "params: " << distr << sycl::endl;
+                });
+            });
+            queue.wait_and_throw();
+
             queue.submit([&](sycl::handler& cgh) {
                 sycl::accessor acc(buffer, cgh, sycl::write_only);
 
@@ -337,36 +488,21 @@ test(sycl::queue& queue)
         }
 
         Distr distr;
-        sum += check_params(distr);
+        status += check_params(distr);
     }
 
-    return sum;
+    return status;
 }
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
 int
 main()
 {
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
 
-    // Catch asynchronous exceptions
-    auto exception_handler = [](sycl::exception_list exceptions) {
-        for (std::exception_ptr const& e : exceptions)
-        {
-            try
-            {
-                std::rethrow_exception(e);
-            }
-            catch (sycl::exception const& e)
-            {
-                std::cout << "Caught asynchronous SYCL exception during generation:\n" << e.what() << std::endl;
-            }
-        }
-    };
-
-    sycl::queue queue(sycl::default_selector{}, exception_handler);
+    sycl::queue queue = TestUtils::get_test_queue();
     std::int32_t err = 0;
 
     // Skip tests if DP is not supported
@@ -639,7 +775,7 @@ main()
 #endif // TEST_LONG_RUN
     EXPECT_TRUE(!err, "Test FAILED");
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
-    return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS);
+    return TestUtils::done(TEST_UNNAMED_LAMBDAS);
 }
\ No newline at end of file
