@@ -19,11 +19,14 @@
 
 #include "support/utils.h"
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
+#    include <oneapi/dpl/random>
+static_assert(ONEDPL_HAS_RANDOM_NUMBERS >= 202409L);
+
 #    include <iostream>
 #    include <vector>
-#    include <CL/sycl.hpp>
-#    include <oneapi/dpl/random>
+
+namespace ex = oneapi::dpl::experimental;
 
 constexpr auto SEED = 777;
 constexpr auto N_GEN = 960;
@@ -39,6 +42,17 @@ constexpr auto RANLUX24_BASE_MIN = 0;
 constexpr auto RANLUX24_BASE_MAX = 16777215;
 constexpr auto RANLUX24_P = 223;
 constexpr auto RANLUX24_R = 23;
+constexpr auto PHILOX4x32_W   = 32;
+constexpr auto PHILOX4x64_W   = 64;
+constexpr auto PHILOX4x32_N   = 4;
+constexpr auto PHILOX4x64_N   = 4;
+constexpr auto PHILOX4x32_R   = 10;
+constexpr auto PHILOX4x64_R   = 10;
+constexpr auto PHILOX4x32_MIN = 0;
+constexpr auto PHILOX4x32_MAX = 0xFFFFFFFF;
+constexpr auto PHILOX4x64_MIN = 0;
+constexpr auto PHILOX4x64_MAX = 0xFFFFFFFFFFFFFFFF;
+constexpr auto PHILOX_SEED = 20111115u;
 
 std::int32_t
 check_params(oneapi::dpl::minstd_rand& engine)
@@ -93,6 +107,46 @@ check_params(oneapi::dpl::ranlux24_vec<N>& engine)
             (engine.max() != RANLUX24_BASE_MAX));
 }
 
+std::int32_t
+check_params(ex::philox4x32& engine)
+{
+    return ((ex::philox4x32::word_size    != PHILOX4x32_W) || 
+            (ex::philox4x32::word_count   != PHILOX4x32_N) ||
+            (ex::philox4x32::round_count  != PHILOX4x32_R) ||
+            (ex::philox4x32::default_seed != PHILOX_SEED)  ||
+            (engine.min() != PHILOX4x32_MIN) || (engine.max() != PHILOX4x32_MAX));
+}
+template <int N>
+std::int32_t
+check_params(ex::philox4x32_vec<N>& engine)
+{
+    return ((ex::philox4x32_vec<N>::word_size    != PHILOX4x32_W) || 
+            (ex::philox4x32_vec<N>::word_count   != PHILOX4x32_N) ||
+            (ex::philox4x32_vec<N>::round_count  != PHILOX4x32_R) ||
+            (ex::philox4x32_vec<N>::default_seed != PHILOX_SEED)  ||
+            (engine.min() != PHILOX4x32_MIN) || (engine.max() != PHILOX4x32_MAX));
+}
+
+std::int32_t
+check_params(ex::philox4x64& engine)
+{
+    return ((ex::philox4x64::word_size    != PHILOX4x64_W) || 
+            (ex::philox4x64::word_count   != PHILOX4x64_N) ||
+            (ex::philox4x64::round_count  != PHILOX4x64_R) ||
+            (ex::philox4x64::default_seed != PHILOX_SEED)  ||
+            (engine.min() != PHILOX4x64_MIN) || (engine.max() != PHILOX4x64_MAX));
+}
+template <int N>
+std::int32_t
+check_params(ex::philox4x64_vec<N>& engine)
+{
+    return ((ex::philox4x64_vec<N>::word_size    != PHILOX4x64_W) || 
+            (ex::philox4x64_vec<N>::word_count   != PHILOX4x64_N) ||
+            (ex::philox4x64_vec<N>::round_count  != PHILOX4x64_R) ||
+            (ex::philox4x64_vec<N>::default_seed != PHILOX_SEED)  ||
+            (engine.min() != PHILOX4x64_MIN) || (engine.max() != PHILOX4x64_MAX));
+}
+
 template <class Engine>
 class
 test_vec
@@ -141,7 +195,7 @@ public:
                         dpstd_acc[offset] = is_inequal;
                     });
                 });
-                auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::write>();
+                auto dpstd_acc = dpstd_buffer.get_host_access(sycl::write_only);
                 for (int i = 0; i < N_GEN; ++i)
                 {
                     sum += dpstd_acc[i];
@@ -217,7 +271,7 @@ public:
                         dpstd_acc[offset] = is_inequal;
                     });
                 });
-                auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::write>();
+                auto dpstd_acc = dpstd_buffer.get_host_access(sycl::write_only);
                 for (int i = 0; i < N_GEN; ++i)
                 {
                     sum += dpstd_acc[i];
@@ -258,24 +312,45 @@ public:
 
         // Random number generation
         {
+            {
+                Engine e1;
+                e1.discard(100);
+
+                std::ostringstream os;
+                os << e1;
+
+                Engine e2;
+
+                std::istringstream in(os.str());
+                in >> e2;
+
+                if (e1 != e2)
+                    sum += 1;
+            }
+
             sycl::buffer<std::int32_t> dpstd_buffer(dpstd_res.data(), dpstd_res.size());
 
             try
             {
                 queue.submit([&](sycl::handler& cgh) {
-                    auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::write>(cgh);
+                    sycl::stream out(1024, 512, cgh);
+                    cgh.single_task<>([=]() {
+                        Engine engine(SEED);
+                        out << "state: " << engine << sycl::endl;
+                    });
+                });
+                queue.wait_and_throw();
+
+                queue.submit([&](sycl::handler& cgh) {
+                    auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::read_write>(cgh);
 
                     cgh.parallel_for<>(sycl::range<1>(N_GEN), [=](sycl::item<1> idx) {
-                        unsigned long long offset = idx.get_linear_id();
+                        unsigned long long offset = idx.get_linear_id() + 1;
                         Engine engine0(SEED);
                         Engine engine1;
                         engine1.seed(SEED);
                         engine0.discard(offset);
-                        engine1.discard(offset);
-                        typename Engine::result_type res0;
-                        res0 = engine0();
-                        typename Engine::result_type res1 = engine1();
-                        if (res0 != res1)
+                        if (engine0 == engine1)
                         {
                             dpstd_acc[offset] = 1;
                         }
@@ -283,9 +358,21 @@ public:
                         {
                             dpstd_acc[offset] = 0;
                         }
+                        engine1.discard(offset);
+                        if (engine0 != engine1)
+                        {
+                            dpstd_acc[offset] += 1;
+                        }
+                        typename Engine::result_type res0;
+                        res0 = engine0();
+                        typename Engine::result_type res1 = engine1();
+                        if (res0 != res1)
+                        {
+                            dpstd_acc[offset] += 1;
+                        }
                     });
                 });
-                auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::write>();
+                auto dpstd_acc = dpstd_buffer.get_host_access(sycl::write_only);
                 for (int i = 0; i < N_GEN; ++i)
                 {
                     sum += dpstd_acc[i];
@@ -327,26 +414,45 @@ public:
 
         // Random number generation
         {
+            {
+                oneapi::dpl::ranlux24 e1;
+                e1.discard(100);
+
+                std::ostringstream os;
+                os << e1;
+
+                oneapi::dpl::ranlux24 e2;
+
+                std::istringstream in(os.str());
+                in >> e2;
+
+                if (e1 != e2)
+                    sum += 1;
+            }
+
             sycl::buffer<std::int32_t> dpstd_buffer(dpstd_res.data(), dpstd_res.size());
 
             try
             {
                 queue.submit([&](sycl::handler& cgh) {
-                    auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::write>(cgh);
+                    sycl::stream out(1024, 512, cgh);
+                    cgh.single_task<>([=]() {
+                        oneapi::dpl::ranlux24 engine(SEED);
+                        out << "state: " << engine << sycl::endl;
+                    });
+                });
+                queue.wait_and_throw();
+
+                queue.submit([&](sycl::handler& cgh) {
+                    auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::read_write>(cgh);
 
                     cgh.parallel_for<>(sycl::range<1>(N_GEN), [=](sycl::item<1> idx) {
-                        unsigned long long offset = idx.get_linear_id();
+                        unsigned long long offset = idx.get_linear_id() + 1;
                         oneapi::dpl::ranlux24 engine0(SEED);
                         oneapi::dpl::ranlux24 engine1;
                         engine1.seed(SEED);
                         engine0.discard(offset);
-                        engine1.discard(offset);
-                        typename oneapi::dpl::ranlux24::result_type res0;
-                        oneapi::dpl::ranlux24 engine(engine1);
-                        auto eng = engine.base();
-                        res0 = engine();
-                        typename oneapi::dpl::ranlux24::result_type res1 = engine1();
-                        if (res0 != res1)
+                        if (engine0 == engine1)
                         {
                             dpstd_acc[offset] = 1;
                         }
@@ -354,9 +460,23 @@ public:
                         {
                             dpstd_acc[offset] = 0;
                         }
+                        engine1.discard(offset);
+                        if (engine0 != engine1)
+                        {
+                            dpstd_acc[offset] += 1;
+                        }
+                        typename oneapi::dpl::ranlux24::result_type res0;
+                        oneapi::dpl::ranlux24 engine(engine1);
+                        auto eng = engine.base();
+                        res0 = engine();
+                        typename oneapi::dpl::ranlux24::result_type res1 = engine1();
+                        if (res0 != res1)
+                        {
+                            dpstd_acc[offset] += 1;
+                        }
                     });
                 });
-                auto dpstd_acc = dpstd_buffer.template get_access<sycl::access::mode::write>();
+                auto dpstd_acc = dpstd_buffer.get_host_access(sycl::write_only);
                 for (int i = 0; i < N_GEN; ++i)
                 {
                     sum += dpstd_acc[i];
@@ -382,29 +502,15 @@ public:
     }
 };
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
 int
 main()
 {
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
-    // Catch asynchronous exceptions
-    auto exception_handler = [](sycl::exception_list exceptions) {
-        for (std::exception_ptr const& e : exceptions)
-        {
-            try
-            {
-                std::rethrow_exception(e);
-            }
-            catch (sycl::exception const& e)
-            {
-                std::cout << "Caught asynchronous SYCL exception during generation:\n" << e.what() << std::endl;
-            }
-        }
-    };
+#if TEST_UNNAMED_LAMBDAS
 
-    sycl::queue queue(sycl::default_selector{}, exception_handler);
+    sycl::queue queue = TestUtils::get_test_queue();
     
     std::int32_t err = 0;
 
@@ -450,7 +556,37 @@ main()
 #endif // TEST_LONG_RUN
     EXPECT_TRUE(!err, "Test FAILED");
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+    std::cout << "---------------------------------------------------" << std::endl;
+    std::cout << "philox_engine<uint_fast32_t, 32, 4, 10,\n"
+                 "0xCD9E8D57, 0x9E3779B9, 0xD2511F53, 0xBB67AE85>" << std::endl;
+    std::cout << "---------------------------------------------------" << std::endl;
+    err += test<ex::philox4x32>{}.run(queue);
+    #if TEST_LONG_RUN
+    err += test_vec<ex::philox4x32_vec<16>>{}.run(queue);
+    err += test_vec<ex::philox4x32_vec<8>>{}.run(queue);
+    err += test_vec<ex::philox4x32_vec<4>>{}.run(queue);
+    err += test_vec<ex::philox4x32_vec<3>>{}.run(queue);
+    err += test_vec<ex::philox4x32_vec<2>>{}.run(queue);
+    err += test_vec<ex::philox4x32_vec<1>>{}.run(queue);
+    #endif // TEST_LONG_RUN
+    EXPECT_TRUE(!err, "Test FAILED");
+    
+    std::cout << "---------------------------------------------------" << std::endl;
+    std::cout << "philox_engine<uint_fast64_t, 64, 4, 10,\n"
+                 "0xCA5A826395121157, 0x9E3779B97F4A7C15, 0xD2E7470EE14C6C93, 0xBB67AE8584CAA73B>" << std::endl;
+    std::cout << "---------------------------------------------------" << std::endl;
+    err += test<ex::philox4x64>{}.run(queue);
+    #if TEST_LONG_RUN
+    err += test_vec<ex::philox4x64_vec<16>>{}.run(queue);
+    err += test_vec<ex::philox4x64_vec<8>>{}.run(queue);
+    err += test_vec<ex::philox4x64_vec<4>>{}.run(queue);
+    err += test_vec<ex::philox4x64_vec<3>>{}.run(queue);
+    err += test_vec<ex::philox4x64_vec<2>>{}.run(queue);
+    err += test_vec<ex::philox4x64_vec<1>>{}.run(queue);
+    #endif // TEST_LONG_RUN
+    EXPECT_TRUE(!err, "Test FAILED");
 
-    return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS);
+#endif // TEST_UNNAMED_LAMBDAS
+
+    return TestUtils::done(TEST_UNNAMED_LAMBDAS);
 }
