@@ -20,8 +20,7 @@
 #include "support/utils.h"
 #include <iostream>
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
-#include <CL/sycl.hpp>
+#if TEST_UNNAMED_LAMBDAS
 #include <limits>
 #include <oneapi/dpl/random>
 #include <math.h>
@@ -70,7 +69,7 @@ test(sycl::queue& queue, double p, int nsamples)
                 oneapi::dpl::bernoulli_distribution<BoolType> distr(p);
 
                 sycl::vec<bool, num_elems> res = distr(engine);
-                res.store(idx.get_linear_id(), acc.get_pointer());
+                res.store(idx.get_linear_id(), acc);
             });
         });
     }
@@ -117,7 +116,7 @@ test_portion(sycl::queue& queue, double p, int nsamples, unsigned int part)
 
                 sycl::vec<bool, num_elems> res = distr(engine, part);
                 for (int i = 0; i < n_elems; ++i)
-                    acc.get_pointer()[idx.get_linear_id() * n_elems + i] = res[i];
+                    acc[idx.get_linear_id() * n_elems + i] = res[i];
             });
         });
         queue.wait_and_throw();
@@ -180,15 +179,15 @@ tests_set_portion(sycl::queue& queue, std::int32_t nsamples, unsigned int part)
     return 0;
 }
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
 int
 main()
 {
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
 
-    sycl::queue queue;
+    sycl::queue queue = TestUtils::get_test_queue();
     // Skip tests if DP is not supported
     if (TestUtils::has_type_support<double>(queue.get_device())) {
         constexpr int nsamples = 100;
@@ -380,7 +379,7 @@ main()
         EXPECT_TRUE(!err, "Test FAILED");
     }
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
-    return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS);
-}
\ No newline at end of file
+    return TestUtils::done(TEST_UNNAMED_LAMBDAS);
+}
