@@ -20,9 +20,8 @@
 #include "support/utils.h"
 #include <iostream>
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
 #include <vector>
-#include <CL/sycl.hpp>
 #include <random>
 #include <oneapi/dpl/random>
 #include "statistics_common.h"
@@ -71,7 +70,7 @@ int test(sycl::queue& queue, oneapi::dpl::internal::element_type_t<IntType> left
                 oneapi::dpl::uniform_int_distribution<IntType> distr(left, right);
 
                 sycl::vec<oneapi::dpl::internal::element_type_t<IntType>, num_elems> res = distr(engine);
-                res.store(idx.get_linear_id(), acc.get_pointer());
+                res.store(idx.get_linear_id(), acc);
             });
         });
         queue.wait();
@@ -115,7 +114,7 @@ int test_portion(sycl::queue& queue, oneapi::dpl::internal::element_type_t<IntTy
 
                 sycl::vec<oneapi::dpl::internal::element_type_t<IntType>, num_elems> res = distr(engine, part);
                 for(int i = 0; i < n_elems; ++i)
-                    acc.get_pointer()[offset + i] = res[i];
+                    acc[offset + i] = res[i];
             });
         });
         queue.wait_and_throw();
@@ -172,13 +171,13 @@ int tests_set_portion(sycl::queue& queue, int nsamples, unsigned int part) {
     return 0;
 }
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
 int main() {
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
 
-    sycl::queue queue;
+    sycl::queue queue = TestUtils::get_test_queue();
     // Skip tests if DP is not supported
     if (TestUtils::has_type_support<double>(queue.get_device())) {
         constexpr int nsamples = 100;
@@ -370,7 +369,7 @@ int main() {
         EXPECT_TRUE(!err, "Test FAILED");
     }
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
-    return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS);
+    return TestUtils::done(TEST_UNNAMED_LAMBDAS);
 }
