@@ -20,9 +20,8 @@
 #include "support/utils.h"
 #include <iostream>
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
 #include <vector>
-#include <CL/sycl.hpp>
 #include <random>
 #include <oneapi/dpl/random>
 
@@ -50,7 +49,7 @@ int test(sycl::queue& queue, oneapi::dpl::internal::element_type_t<UIntType> see
                 oneapi::dpl::linear_congruential_engine<UIntType, a, c, m> engine(seed, offset);
 
                 sycl::vec<oneapi::dpl::internal::element_type_t<UIntType>, num_elems> res = engine();
-                res.store(idx.get_linear_id(), dpstd_acc.get_pointer());
+                res.store(idx.get_linear_id(), dpstd_acc);
             });
         });
         queue.wait();
@@ -106,7 +105,7 @@ int test_portion(sycl::queue& queue, oneapi::dpl::internal::element_type_t<UIntT
 
                 auto res = engine(part);
                 for(int i = 0; i < n_elems; ++i)
-                    dpstd_acc.get_pointer()[offset + i] = res[i];
+                    dpstd_acc[offset + i] = res[i];
             });
         });
         queue.wait();
@@ -190,13 +189,13 @@ int tests_set_portion(sycl::queue& queue, int nsamples, unsigned int part) {
     return 0;
 }
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
 int main() {
 
-#if TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#if TEST_UNNAMED_LAMBDAS
 
-    sycl::queue queue;
+    sycl::queue queue = TestUtils::get_test_queue();
 
     constexpr int nsamples = 100;
     int err = 0;
@@ -345,7 +344,7 @@ int main() {
     EXPECT_TRUE(!err, "Test FAILED");
 #endif // TEST_LONG_RUN
 
-#endif // TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS
+#endif // TEST_UNNAMED_LAMBDAS
 
-    return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT && TEST_UNNAMED_LAMBDAS);
+    return TestUtils::done(TEST_UNNAMED_LAMBDAS);
 }
